/*
!C******************************************************************
!Description:
	Functions to handle output to HDF files.

!Revision History:
!Team-unique Header:
!References and Credits:

	W. Schroeder, University of Maryland
	W. Walsh, University of Maryland

!Design Notes:

!END*************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <unistd.h>

#include "hdf5.h" // delete
#include "hdf.h"
#include "mfhdf.h"

#include "const.h"
#include "types.h"

#include "output.h"
#include "input.h"
#include "error.h"
#include "misc.h"
#include "version.h"

#include <time.h>



extern int write_tb, write_ref;
static int set_dimensions(int32 sds_id, char *dimname1, char *dimname2);
static int write_band_scan(int32 sd_id, int32 sds_index, int32 start[2],
	int32 edges[2], void *buff);
static char *satstr(enum Satellite sat);


/* Sets up and fills in the OutputInfo struct */
/* Sets chuncking */
/* Writes a few global attributes*/
int InitOutputHDF(char *outfile, int nscans, OutputInfo *out_info, char input_files[8][100], int write_ascii, int write_hdf, char *ascii_file, hid_t sd_gitco){

	int32 sd_id_fire;
	char msg[MAX_MSG_LEN];

	/* Create output file id */
	if ((sd_id_fire = SDstart(outfile, DFACC_CREATE)) == FAIL) {
		(void) error(ERR_HDF_QUIET);
		(void) sprintf(msg, "Unable to open HDF output file \"%s\".", outfile);
		errmsg(msg);
		exit(3);
		}

	out_info->outfile = outfile;
	out_info->write_hdf = write_hdf;

	/* Set up ascii struct */
	if (write_ascii){
		out_info->write_ascii = 1;
		out_info->ascii_ptr = NULL;
		out_info->ascii_name = ascii_file;
		out_info->svi01_name = input_files[0];
	}else{
		out_info->write_ascii = 0;
	}
	
	int32 sds_id;
	int rt;

	int32 dim_sizes[2];
	char *dimname1, *dimname2;

	int sds_index = 0;

	int32 n_values;
	
	char units_str[1000];
	char *units;
	units = &units_str;
	char *legend;
	

	HDF_CHUNK_DEF cdef;


	/* ========== Level 2 Fire Product ========== */
	out_info->sd_id_fire = sd_id_fire;

	dim_sizes[0] = nscans * LINES_PER_SCAN_375M;
	dim_sizes[1] = SAMPLES_375M;

	dimname1 = "number_of_scan_lines";
	dimname2 = "pixels_per_scan_line";

	cdef.chunk_lengths[0] = cdef.comp.chunk_lengths[0] = LINES_PER_SCAN_375M;
	cdef.chunk_lengths[1] = cdef.comp.chunk_lengths[1] = SAMPLES_375M;
	cdef.comp.comp_type = COMP_CODE_DEFLATE;
	cdef.comp.cinfo.deflate.level = 4;

	/* ----- Global Attr ----- */

	n_values = strlen(input_files[0]);
	if (SDsetattr(sd_id_fire, "SVI01", DFNT_CHAR8, n_values, input_files[0]))
		return error(ERR_HDF);

	n_values = strlen(input_files[1]);
	if (SDsetattr(sd_id_fire, "SVI02", DFNT_CHAR8, n_values, input_files[1]))
		return error(ERR_HDF);

	n_values = strlen(input_files[2]);
	if (SDsetattr(sd_id_fire, "SVI03", DFNT_CHAR8, n_values, input_files[2]))
		return error(ERR_HDF);

	n_values = strlen(input_files[3]);
	if (SDsetattr(sd_id_fire, "SVI04", DFNT_CHAR8, n_values, input_files[3]))
		return error(ERR_HDF);

	n_values = strlen(input_files[4]);
	if (SDsetattr(sd_id_fire, "SVI05", DFNT_CHAR8, n_values, input_files[4]))
		return error(ERR_HDF);

	n_values = strlen(input_files[5]);
	if (SDsetattr(sd_id_fire, "GITCO", DFNT_CHAR8, n_values, input_files[5]))
		return error(ERR_HDF);

	if (strcmp(input_files[6],"NULL") != 0){
		n_values = strlen(input_files[6]);
		if (SDsetattr(sd_id_fire, "SVM13_agg", DFNT_CHAR8, n_values, input_files[6]))
			return error(ERR_HDF);

		n_values = strlen(input_files[7]);
		if (SDsetattr(sd_id_fire, "SVM13_unagg", DFNT_CHAR8, n_values, input_files[7]))
			return error(ERR_HDF);
	}

	char *temp_ptr = PROCESS_VERSION_NUMBER;
	n_values = strlen(temp_ptr);
	if (SDsetattr(sd_id_fire, "ProcessVersionNumber", DFNT_CHAR8, n_values, temp_ptr))
		return error(ERR_HDF);

	temp_ptr = __DATE__;
	n_values = strlen(temp_ptr);
	if (SDsetattr(sd_id_fire, "ExecutableCreationDate", DFNT_CHAR8, n_values, temp_ptr))
		return error(ERR_HDF);

	temp_ptr = __TIME__;
	n_values = strlen(temp_ptr);
	if (SDsetattr(sd_id_fire, "ExecutableCreationTime", DFNT_CHAR8, n_values, temp_ptr))
		return error(ERR_HDF);


	char Sat[4];
	strcpy(Sat, "NPP");
	temp_ptr = &Sat;
	n_values = strlen(temp_ptr);
	if (SDsetattr(sd_id_fire, "Satellite", DFNT_CHAR8, n_values, temp_ptr))
		return error(ERR_HDF);

	char *s;
	s = make_uname();
	n_values = strlen(s);
	if (SDsetattr(sd_id_fire, "SystemID", DFNT_CHAR8, n_values, s))
		return error(ERR_HDF);

	/***** GRing Lat/Lon & Bounding Coords *****/
	/* GRing Lat/Lon */
	char sds_str[100];
	char num_str[3];
	int Granules;
	int *Granules_ptr;
	Granules_ptr = &Granules;
	char gringlat[2000];
	char gringlon[2000];
	char temp_str[300];
	float *tempf_ptr;
	int i;
	float array[8];
	float *array_ptr;
	array_ptr = &array;
	float single_float;
	float *float_ptr;
	float_ptr = &single_float;
	float north_coord, south_coord, east_coord, west_coord;

	/* Getting number of granules */
	hid_t sds_ind = H5Dopen1(sd_gitco, "/Data_Products/VIIRS-IMG-GEO-TC/VIIRS-IMG-GEO-TC_Aggr");
	hid_t attr_id = H5Aopen(sds_ind, "AggregateNumberGranules", H5P_DEFAULT);
	hid_t err = H5Aread(attr_id, H5T_NATIVE_INT, Granules_ptr);
	
	for(i=0;i<(*Granules_ptr);i++){
		/* Opening unagg granule #i */
		strcpy(sds_str,"/Data_Products/VIIRS-IMG-GEO-TC/VIIRS-IMG-GEO-TC_Gran_");
		sprintf(num_str, "%d", i);
		strcat(sds_str, num_str);
		sds_ind = H5Dopen1(sd_gitco, sds_str);

		/* Open and Read G-Ring Lat */
		attr_id = H5Aopen(sds_ind, "G-Ring_Latitude", H5P_DEFAULT);
		err = H5Aread(attr_id, H5T_NATIVE_FLOAT, array_ptr);

		/* Save G-Ring Lat */
		if(i == 0){
			sprintf(gringlat,"TS %d: %f, %f, %f, %f, %f, %f, %f, %f", i, array_ptr[0], array_ptr[1], array_ptr[2], array_ptr[3], array_ptr[4], array_ptr[5], array_ptr[6], array_ptr[7]);
		}else{
			strcat(gringlat, "; ");
			sprintf(temp_str,"TS %d: %f, %f, %f, %f, %f, %f, %f, %f", i, array_ptr[0], array_ptr[1], array_ptr[2], array_ptr[3], array_ptr[4], array_ptr[5], array_ptr[6], array_ptr[7]);
			strcat(gringlat, temp_str);
		}


		/* Open and Read G-Ring Lon */
		attr_id = H5Aopen(sds_ind, "G-Ring_Longitude", H5P_DEFAULT);
		err = H5Aread(attr_id, H5T_NATIVE_FLOAT, array_ptr);

		/* Save G-Ring Lon */
		if(i == 0){
			sprintf(gringlon,"TS %d: %f, %f, %f, %f, %f, %f, %f, %f", i, array_ptr[0], array_ptr[1], array_ptr[2], array_ptr[3], array_ptr[4], array_ptr[5], array_ptr[6], array_ptr[7]);
		}else{
			strcat(gringlon, "; ");
			sprintf(temp_str,"TS %d: %f, %f, %f, %f, %f, %f, %f, %f", i, array_ptr[0], array_ptr[1], array_ptr[2], array_ptr[3], array_ptr[4], array_ptr[5], array_ptr[6], array_ptr[7]);
			strcat(gringlon, temp_str);
		}

		/* Open and Read North Coord */
		attr_id = H5Aopen(sds_ind, "North_Bounding_Coordinate", H5P_DEFAULT);
		err = H5Aread(attr_id, H5T_NATIVE_FLOAT, float_ptr);
		if(i == 0){
			north_coord = *float_ptr;
		}else if(north_coord < *float_ptr){
			north_coord = *float_ptr;
		}

		/* Open and Read South Coord */
		attr_id = H5Aopen(sds_ind, "South_Bounding_Coordinate", H5P_DEFAULT);
		err = H5Aread(attr_id, H5T_NATIVE_FLOAT, float_ptr);
		if(i == 0){
			south_coord = *float_ptr;
		}else if(south_coord > *float_ptr){
			south_coord = *float_ptr;
		}

		/* Open and Read East Coord */
		attr_id = H5Aopen(sds_ind, "East_Bounding_Coordinate", H5P_DEFAULT);
		err = H5Aread(attr_id, H5T_NATIVE_FLOAT, float_ptr);
		if(i == 0){
			east_coord = *float_ptr;
		}else if(east_coord < *float_ptr){
			east_coord = *float_ptr;
		}

		/* Open and Read West Coord */
		attr_id = H5Aopen(sds_ind, "West_Bounding_Coordinate", H5P_DEFAULT);
		err = H5Aread(attr_id, H5T_NATIVE_FLOAT, float_ptr);
		if(i == 0){
			west_coord = *float_ptr;
		}else if(west_coord > *float_ptr){
			west_coord = *float_ptr;
		}


	}
	temp_ptr = &gringlat;
	n_values = strlen(temp_ptr);
	if (SDsetattr(sd_id_fire, "Unagg_GRingLatitude", DFNT_CHAR8, n_values, temp_ptr))
		return error(ERR_HDF);
	
	temp_ptr = &gringlon;
	n_values = strlen(temp_ptr);
	if (SDsetattr(sd_id_fire, "Unagg_GRingLongitude", DFNT_CHAR8, n_values, temp_ptr))
		return error(ERR_HDF);
	
	tempf_ptr = &north_coord;
	if (SDsetattr(sd_id_fire, "North_Bounding_Coordinate", DFNT_FLOAT, 1, tempf_ptr))
		return error(ERR_HDF);

	tempf_ptr = &south_coord;
	if (SDsetattr(sd_id_fire, "South_Bounding_Coordinate", DFNT_FLOAT, 1, tempf_ptr))
		return error(ERR_HDF);

	tempf_ptr = &east_coord;
	if (SDsetattr(sd_id_fire, "East_Bounding_Coordinate", DFNT_FLOAT, 1, tempf_ptr))
		return error(ERR_HDF);

	tempf_ptr = &west_coord;
	if (SDsetattr(sd_id_fire, "West_Bounding_Coordinate", DFNT_FLOAT, 1, tempf_ptr))
		return error(ERR_HDF);
	/****			****/

	/*** Start and End Time Attributes ***/
	/* Getting the granule time */
	char start_time[60];
	char end_time[60];
	char temp[30];
	int len, offset, date_pos;
	char year[5], month[3], day[3], hour[3], minute[3], sec[3], sec_fraction[2];

	offset = 67;
	len = strlen(input_files[0]);
	date_pos = len - offset;
	strncpy(temp, input_files[0] + date_pos, 26);
	temp[26] = '\0';

	strncpy(year, temp, 4);
	year[4] = '\0';
	strncpy(month, temp+4, 2);
	month[2] = '\0';
	strncpy(day, temp+6, 2);
	day[2] = '\0';
	
	strncpy(hour, temp+10, 2);
	hour[2] = '\0';
	strncpy(minute, temp+12, 2);
	minute[2] = '\0';
	strncpy(sec, temp+14, 2);
	sec[2] = '\0';
	strncpy(sec_fraction, temp+16, 1);
	sec_fraction[1] = '\0';

	sprintf(start_time, "%s-%s-%s %s:%s:%s.%s", (char *)&year, (char *)&month, (char *)&day, (char *)&hour, (char *)&minute, (char *)&sec, (char *)&sec_fraction);

	strncpy(hour, temp+19, 2);
	hour[2] = '\0';
	strncpy(minute, temp+21, 2);
	minute[2] = '\0';
	strncpy(sec, temp+23, 2);
	sec[2] = '\0';
	strncpy(sec_fraction, temp+25, 1);
	sec_fraction[1] = '\0';

	sprintf(end_time, "%s-%s-%s %s:%s:%s.%s", (char *)&year, (char *)&month, (char *)&day, (char *)&hour, (char *)&minute, (char *)&sec, (char *)&sec_fraction);

	temp_ptr = &start_time;
	n_values = strlen(temp_ptr);
	if (SDsetattr(sd_id_fire, "StartTime", DFNT_CHAR8, n_values, temp_ptr))
		return error(ERR_HDF);

	temp_ptr = &end_time;
	n_values = strlen(temp_ptr);
	if (SDsetattr(sd_id_fire, "EndTime", DFNT_CHAR8, n_values, temp_ptr))
		return error(ERR_HDF);
	/***		***/


	/* ----- fire mask ----- */

	out_info->sds_index_mask = sds_index++;
	if ((sds_id = SDcreate(sd_id_fire, "fire mask", DFNT_UINT8, 2,
		dim_sizes)) == FAIL) return error(ERR_HDF);

	if (SDsetchunk(sds_id, cdef, HDF_CHUNK | HDF_COMP) == FAIL)
		return error(ERR_HDF);

	if (set_dimensions(sds_id, dimname1, dimname2))
		return error(ERR_HDF);
legend =
	"\n0 not-processed (non-zero QF)\n"
	"1 bowtie\n"
	"2 glint\n"
	"3 water\n"
	"4 clouds\n"
	"5 clear land\n"
	"6 unclassified fire pixel\n"
	"7 low confidence fire pixel\n"
	"8 nominal confidence fire pixel\n"
	"9 high confidence fire pixel\n";

	/* Add attributes */
	n_values = strlen(legend);
	if (SDsetattr(sds_id, "legend", DFNT_CHAR8, n_values, legend))
		return error(ERR_HDF);

	if (SDendaccess(sds_id) == FAIL) return error(ERR_HDF);

	/* ----- QA ----- */

	out_info->sds_index_qa = sds_index++;
	if ((sds_id = SDcreate(sd_id_fire, "algorithm QA", DFNT_UINT32, 2,
		dim_sizes)) == FAIL)
		return error(ERR_HDF);

	if (SDsetchunk(sds_id, cdef, HDF_CHUNK | HDF_COMP) == FAIL)
		return error(ERR_HDF);

	if (set_dimensions(sds_id, dimname1, dimname2))
		return error(ERR_HDF);

	/* Add attributes */
	strcpy(units, "bit field");
	n_values = strlen(units);
	if (SDsetattr(sds_id, "units", DFNT_CHAR8, n_values, units))
		return error(ERR_HDF);

	if (SDendaccess(sds_id) == FAIL) return error(ERR_HDF);

	return 0;
}

/* Top function for writing data to hdf/ascii files */
int WriteOutput(OutputInfo *out_info, InputInfo *in_info, Juggernaut *jugg, FireData *fire_data){

	/* Writes jugg->pixflag.mask to HDF output */
	WriteMask(out_info, jugg);

	/* Populates empty FireData fields from the Juggernaut */
	if( (out_info->write_ascii || out_info->write_hdf) && fire_data->index != 0 ) PopulateFireData(jugg, fire_data);

	/* Sort FireData fields */
	if( (out_info->write_ascii || out_info->write_hdf) && fire_data->index != 0 ) SortFireData(fire_data);

	/* Write FireData info to hdf SDSs */
	if(out_info->write_hdf) WriteFireDataSDSs(out_info, fire_data);

	/* Write FireData info to ascii file */
	if(out_info->write_ascii && fire_data->index != 0) WriteFireDataASCII(out_info, in_info, jugg, fire_data);

	/* Write post run global attributes */
	WriteGlobalAttr(out_info, jugg, fire_data);
}

void SortFireData(FireData *fire_data){
	uint8 Mask; 
	float Lon;
	float Lat;
	uint16 Row;
	uint16 Col;
	float T4;
	float T5;
	uint16 Window_Size;
	float SolZenAng;
	float SolAzAng;
	float ViewZenAng;
	float ViewAzAng;
	uint8 Day;
	float mean_T4;
	float mean_T5;
	float MAD_T4;
	float MAD_T5;
	float mean_Dif45;
	float MAD_Dif45;
	uint16 Adj_Cloud;
	uint16 Adj_Water;
	float FRP;



	int x, y;
	int n = fire_data->index;
	int change_made_flag = 0;
	for(x=0; x<n; x++){
		change_made_flag = 0;
		for(y=0; y<n-1; y++){
			if(fire_data->Row[y] > fire_data->Row[y+1]){
				change_made_flag = 1;
				/* Save [y+1] */
				Mask = fire_data->Mask[y+1];
				Lon = fire_data->Lon[y+1];
				Lat = fire_data->Lat[y+1];
				Row = fire_data->Row[y+1];
				Col = fire_data->Col[y+1];
				T4 = fire_data->T4[y+1];
				T5 = fire_data->T5[y+1];
				Window_Size = fire_data->Window_Size[y+1];
				SolZenAng = fire_data->SolZenAng[y+1];
				SolAzAng = fire_data->SolAzAng[y+1];
				ViewZenAng = fire_data->ViewZenAng[y+1];
				ViewAzAng = fire_data->ViewAzAng[y+1];
				Day = fire_data->Day[y+1];
				mean_T4 = fire_data->mean_T4[y+1];
				mean_T5 = fire_data->mean_T5[y+1];
				MAD_T4 = fire_data->MAD_T4[y+1];
				MAD_T5 = fire_data->MAD_T5[y+1];
				mean_Dif45 = fire_data->mean_Dif45[y+1];
				MAD_Dif45 = fire_data->MAD_Dif45[y+1];
				Adj_Cloud = fire_data->Adj_Cloud[y+1];
				Adj_Water = fire_data->Adj_Water[y+1];
				FRP = fire_data->FRP[y+1];
				

				/* [y+1] = [y]*/
				fire_data->Mask[y+1] = fire_data->Mask[y];
				fire_data->Lon[y+1] = fire_data->Lon[y];
				fire_data->Lat[y+1] = fire_data->Lat[y];
				fire_data->Row[y+1] = fire_data->Row[y];
				fire_data->Col[y+1] = fire_data->Col[y];
				fire_data->T4[y+1] = fire_data->T4[y];
				fire_data->T5[y+1] = fire_data->T5[y];
				fire_data->Window_Size[y+1] = fire_data->Window_Size[y];
				fire_data->SolZenAng[y+1] = fire_data->SolZenAng[y];
				fire_data->SolAzAng[y+1] = fire_data->SolAzAng[y];
				fire_data->ViewZenAng[y+1] = fire_data->ViewZenAng[y];
				fire_data->ViewAzAng[y+1] = fire_data->ViewAzAng[y];
				fire_data->Day[y+1] = fire_data->Day[y];
				fire_data->mean_T4[y+1] = fire_data->mean_T4[y];
				fire_data->mean_T5[y+1] = fire_data->mean_T5[y];
				fire_data->MAD_T4[y+1] = fire_data->MAD_T4[y];
				fire_data->MAD_T5[y+1] = fire_data->MAD_T5[y];
				fire_data->mean_Dif45[y+1] = fire_data->mean_Dif45[y];
				fire_data->MAD_Dif45[y+1] = fire_data->MAD_Dif45[y];
				fire_data->Adj_Cloud[y+1] = fire_data->Adj_Cloud[y];
				fire_data->Adj_Water[y+1] = fire_data->Adj_Water[y];
				fire_data->FRP[y+1]= fire_data->FRP[y];
				

				/* [y] = [y+1]*/
				fire_data->Mask[y] = Mask;
				fire_data->Lon[y] = Lon;
				fire_data->Lat[y] = Lat;
				fire_data->Row[y] = Row;
				fire_data->Col[y] = Col;
				fire_data->T4[y] = T4;
				fire_data->T5[y] = T5;
				fire_data->Window_Size[y] = Window_Size;
				fire_data->SolZenAng[y] = SolZenAng;
				fire_data->SolAzAng[y] = SolAzAng;
				fire_data->ViewZenAng[y] = ViewZenAng;
				fire_data->ViewAzAng[y] = ViewAzAng;
				fire_data->Day[y] = Day;
				fire_data->mean_T4[y] = mean_T4;
				fire_data->mean_T5[y] = mean_T5;
				fire_data->MAD_T4[y] = MAD_T4;
				fire_data->MAD_T5[y] = MAD_T5;
				fire_data->mean_Dif45[y] = mean_Dif45;
				fire_data->MAD_Dif45[y] = MAD_Dif45;
				fire_data->Adj_Cloud[y] = Adj_Cloud;
				fire_data->Adj_Water[y] = Adj_Water;
				fire_data->FRP[y]= FRP;
				
			}
		}
		if(change_made_flag == 0) break;
	}

}

/* Write FireData info to ascii file */
void WriteFireDataASCII(OutputInfo *out_info, InputInfo *in_info, Juggernaut *jugg, FireData *fire_data){
	/* Needed to convert spacecraft time to UTC */
	//int epoch_difference = 378691200;
	
	char scan_str[40];
	int scan_num;
	char Scan_Time[20];
	int f;
	char Confidence[8];
	char Period[6];

	double fire_secs;
	time_t fire_secs_since_epoch;
	time_t scan_t;
	struct tm *scan_ptr;
	double scan0_secs;
	double beginning_secs;

	/* Get beginning time in seconds since 1970 */
	

	char year[5];
	char month[3];
	char day[3];
	char hour[3];
	char minute[3];
	char second[3];

	strncpy(&year, &in_info->BeginningDate[0], 4);
	strncpy(&month, &in_info->BeginningDate[4], 2);
	strncpy(&day, &in_info->BeginningDate[6], 2);
	strncpy(&hour, &in_info->BeginningTime[0], 2);
	strncpy(&minute, &in_info->BeginningTime[2], 2);
	strncpy(&second, &in_info->BeginningTime[4], 2);
	year[4] = '\0';
	month[2] = '\0';
	day[2] = '\0';
	hour[2] = '\0';
	minute[2] = '\0';
	second[2] = '\0';

	struct tm Beginning;
	Beginning.tm_year = atoi(year) - 1900;
	Beginning.tm_mon = atoi(month) - 1;
	Beginning.tm_mday = atoi(day);
	Beginning.tm_hour = atoi(hour);
	Beginning.tm_min = atoi(minute);
	Beginning.tm_sec = atoi(second);
	Beginning.tm_isdst = 0;

	putenv("TZ=UTC");
	beginning_secs = mktime(&Beginning);
	/*******************/
	
	/* midtime[0] seconds since 1957 */
	scan0_secs = jugg->MidTime[0]/1000000;



	/* Writes header in ascii file */
	InitOutputASCII(out_info, fire_data);




	for(f=0;f<fire_data->index;f++){
		if(fire_data->Mask[f] == 8 || fire_data->Mask[f] == 9){
			strcpy(Confidence, "nominal");
		}else if(fire_data->Mask[f] == 7){
			strcpy(Confidence, "low");
		}
		
		if(fire_data->Day[f] == 1){
			strcpy(Period, "day");
		}else{
			strcpy(Period, "night");
		}
		
		/* Scan Time */
		scan_num = fire_data->Row[f] / 32;
		fire_secs = jugg->MidTime[scan_num]/1000000;
		fire_secs_since_epoch = beginning_secs + (fire_secs - scan0_secs);
		scan_ptr = gmtime(&fire_secs_since_epoch);	
		strftime(scan_str, sizeof(scan_str), "%Y%m%d_%H:%M:%S\0",scan_ptr);
		strcpy(Scan_Time, scan_str);



		fprintf(out_info->ascii_ptr, "%27s,%18s,%11.5f,%10.5f,%7d,%7d,%9.1f,%9.1f,%11s,%7s\n", &fire_data->Granule, &Scan_Time, fire_data->Lon[f], fire_data->Lat[f], fire_data->Col[f], fire_data->Row[f], fire_data->T4[f], fire_data->T5[f], &Confidence, &Period);
	}
	fflush(out_info->ascii_ptr);
}

/* Write FireData info to hdf SDSs */
/* (could shorten by creating SDS writing module) */
void WriteSDS(){


}

void WriteFireDataSDSs(OutputInfo *out_info, FireData *fire_data){

	int32 sds_id;
	int32 dim_id;

	int32 dim_sizes[2];
	char *dimname1, *dimname2;

	int sds_index = out_info->sds_index_qa;

	int32 n_values;
	
	int start[1];
	int edges[1];

	char units_str[1000];
	char *units;
	units = &units_str;
	char long_name_str[1000];
	char *long_name;
	long_name = &long_name_str;
	char *legend;
	

	HDF_CHUNK_DEF cdef;

	/* Common */
	dim_sizes[0] = fire_data->index;
	dimname1 = "number_of_fires";

	cdef.chunk_lengths[0] = cdef.comp.chunk_lengths[0] = fire_data->index;
	cdef.comp.comp_type = COMP_CODE_DEFLATE;
	cdef.comp.cinfo.deflate.level = 4;

	start[0] = 0;
	edges[0] = dim_sizes[0];

	/* --- Row --- */
	/* Init */
	out_info->sds_index_row = sds_index++;
	if ((sds_id = SDcreate(out_info->sd_id_fire, "FP_line", DFNT_UINT16, 1,
		dim_sizes)) == FAIL)
		return error(ERR_HDF);

	if (dim_sizes[0] != 0){
		if (SDsetchunk(sds_id, cdef, HDF_CHUNK | HDF_COMP) == FAIL)
			return error(ERR_HDF);

		if ((dim_id = SDgetdimid(sds_id, 0)) == FAIL) return ERR_HDF;
		if (SDsetdimname(dim_id, dimname1) == FAIL) return ERR_HDF;

		/* Write */	
		if (SDwritedata(sds_id, start, NULL, edges, fire_data->Row) == FAIL)
			return ERR_HDF;
	}

	/* Add attributes */
	strcpy(long_name, "granule line of fire pixel");
	n_values = strlen(long_name);
	if (SDsetattr(sds_id, "long_name", DFNT_CHAR8, n_values, long_name))
		return error(ERR_HDF);

	if (SDendaccess(sds_id) == FAIL) return error(ERR_HDF);
	/* --- --- */

	/* --- Col --- */
	/* Init */
	out_info->sds_index_col = sds_index++;
	if ((sds_id = SDcreate(out_info->sd_id_fire, "FP_sample", DFNT_UINT16, 1,
		dim_sizes)) == FAIL)
		return error(ERR_HDF);

	if (dim_sizes[0] != 0){
		if (SDsetchunk(sds_id, cdef, HDF_CHUNK | HDF_COMP) == FAIL)
			return error(ERR_HDF);

		if ((dim_id = SDgetdimid(sds_id, 0)) == FAIL) return ERR_HDF;
		if (SDsetdimname(dim_id, dimname1) == FAIL) return ERR_HDF;

		/* Write */	
		if (SDwritedata(sds_id, start, NULL, edges, fire_data->Col) == FAIL)
			return ERR_HDF;
	}

	/* Add attributes */
	strcpy(long_name, "granule sample of fire pixel");
	n_values = strlen(long_name);
	if (SDsetattr(sds_id, "long_name", DFNT_CHAR8, n_values, long_name))
		return error(ERR_HDF);


	if (SDendaccess(sds_id) == FAIL) return error(ERR_HDF);
	/* --- --- */

	/* --- Lat --- */
	/* Init */
	out_info->sds_index_lat = sds_index++;
	if ((sds_id = SDcreate(out_info->sd_id_fire, "FP_latitude", DFNT_FLOAT32, 1,
		dim_sizes)) == FAIL)
		return error(ERR_HDF);

	if (dim_sizes[0] != 0){
		if (SDsetchunk(sds_id, cdef, HDF_CHUNK | HDF_COMP) == FAIL)
			return error(ERR_HDF);

		if ((dim_id = SDgetdimid(sds_id, 0)) == FAIL) return ERR_HDF;
		if (SDsetdimname(dim_id, dimname1) == FAIL) return ERR_HDF;

		/* Write */	
		if (SDwritedata(sds_id, start, NULL, edges, fire_data->Lat) == FAIL)
			return ERR_HDF;
	}

	/* Add attributes */
	strcpy(long_name, "latitude of fire pixel");
	n_values = strlen(long_name);
	if (SDsetattr(sds_id, "long_name", DFNT_CHAR8, n_values, long_name))
		return error(ERR_HDF);

	strcpy(units, "degrees");
	n_values = strlen(units);
	if (SDsetattr(sds_id, "units", DFNT_CHAR8, n_values, units))
		return error(ERR_HDF);

	if (SDendaccess(sds_id) == FAIL) return error(ERR_HDF);
	/* --- --- */

	/* --- Lon --- */
	/* Init */
	out_info->sds_index_lon = sds_index++;
	if ((sds_id = SDcreate(out_info->sd_id_fire, "FP_longitude", DFNT_FLOAT32, 1,
		dim_sizes)) == FAIL)
		return error(ERR_HDF);

	if (dim_sizes[0] != 0){
		if (SDsetchunk(sds_id, cdef, HDF_CHUNK | HDF_COMP) == FAIL)
			return error(ERR_HDF);

		if ((dim_id = SDgetdimid(sds_id, 0)) == FAIL) return ERR_HDF;
		if (SDsetdimname(dim_id, dimname1) == FAIL) return ERR_HDF;

		/* Write */	
		if (SDwritedata(sds_id, start, NULL, edges, fire_data->Lon) == FAIL)
			return ERR_HDF;
	}

	/* Add attributes */
	strcpy(long_name, "longitude of fire pixel");
	n_values = strlen(long_name);
	if (SDsetattr(sds_id, "long_name", DFNT_CHAR8, n_values, long_name))
		return error(ERR_HDF);

	strcpy(units, "degrees");
	n_values = strlen(units);
	if (SDsetattr(sds_id, "units", DFNT_CHAR8, n_values, units))
		return error(ERR_HDF);

	if (SDendaccess(sds_id) == FAIL) return error(ERR_HDF);
	/* --- --- */

	/* --- BT4 --- */
	/* Init */
	out_info->sds_index_T4 = sds_index++;
	if ((sds_id = SDcreate(out_info->sd_id_fire, "FP_T4", DFNT_FLOAT32, 1,
		dim_sizes)) == FAIL)
		return error(ERR_HDF);

	if (dim_sizes[0] != 0){
		if (SDsetchunk(sds_id, cdef, HDF_CHUNK | HDF_COMP) == FAIL)
			return error(ERR_HDF);

		if ((dim_id = SDgetdimid(sds_id, 0)) == FAIL) return ERR_HDF;
		if (SDsetdimname(dim_id, dimname1) == FAIL) return ERR_HDF;

		/* Write */	
		if (SDwritedata(sds_id, start, NULL, edges, fire_data->T4) == FAIL)
			return ERR_HDF;
	}

	/* Add attributes */
	strcpy(long_name, "I04 brightness temperature of fire pixel");
	n_values = strlen(long_name);
	if (SDsetattr(sds_id, "long_name", DFNT_CHAR8, n_values, long_name))
		return error(ERR_HDF);

	strcpy(units, "kelvins");
	n_values = strlen(units);
	if (SDsetattr(sds_id, "units", DFNT_CHAR8, n_values, units))
		return error(ERR_HDF);

	if (SDendaccess(sds_id) == FAIL) return error(ERR_HDF);
	/* --- --- */


	/* --- BT5 --- */
	/* Init */
	out_info->sds_index_T5 = sds_index++;
	if ((sds_id = SDcreate(out_info->sd_id_fire, "FP_T5", DFNT_FLOAT32, 1,
		dim_sizes)) == FAIL)
		return error(ERR_HDF);

	if (dim_sizes[0] != 0){
		if (SDsetchunk(sds_id, cdef, HDF_CHUNK | HDF_COMP) == FAIL)
			return error(ERR_HDF);

		if ((dim_id = SDgetdimid(sds_id, 0)) == FAIL) return ERR_HDF;
		if (SDsetdimname(dim_id, dimname1) == FAIL) return ERR_HDF;

		/* Write */
		if (SDwritedata(sds_id, start, NULL, edges, fire_data->T5) == FAIL)
			return ERR_HDF;
	}

	/* Add attributes */
	strcpy(long_name, "I05 brightness temperature of fire pixel");
	n_values = strlen(long_name);
	if (SDsetattr(sds_id, "long_name", DFNT_CHAR8, n_values, long_name))
		return error(ERR_HDF);

	strcpy(units, "kelvins");
	n_values = strlen(units);
	if (SDsetattr(sds_id, "units", DFNT_CHAR8, n_values, units))
		return error(ERR_HDF);

	if (SDendaccess(sds_id) == FAIL) return error(ERR_HDF);
	/* --- --- */


	/* --- mean_T4 --- */
	/* Init */
	out_info->sds_index_MeanT4 = sds_index++;
	if ((sds_id = SDcreate(out_info->sd_id_fire, "FP_MeanT4", DFNT_FLOAT32, 1,
		dim_sizes)) == FAIL)
		return error(ERR_HDF);

	if (dim_sizes[0] != 0){
		if (SDsetchunk(sds_id, cdef, HDF_CHUNK | HDF_COMP) == FAIL)
			return error(ERR_HDF);

		if ((dim_id = SDgetdimid(sds_id, 0)) == FAIL) return ERR_HDF;
		if (SDsetdimname(dim_id, dimname1) == FAIL) return ERR_HDF;

		/* Write */	
		if (SDwritedata(sds_id, start, NULL, edges, fire_data->mean_T4) == FAIL)
			return ERR_HDF;
	}

	/* Add attributes */
	strcpy(long_name, "I04 brightness temperature of background");
	n_values = strlen(long_name);
	if (SDsetattr(sds_id, "long_name", DFNT_CHAR8, n_values, long_name))
		return error(ERR_HDF);

	strcpy(units, "kelvins");
	n_values = strlen(units);
	if (SDsetattr(sds_id, "units", DFNT_CHAR8, n_values, units))
		return error(ERR_HDF);

	if (SDendaccess(sds_id) == FAIL) return error(ERR_HDF);
	/* --- --- */


	/* --- mean_T5 --- */
	/* Init */
	out_info->sds_index_MeanT5 = sds_index++;
	if ((sds_id = SDcreate(out_info->sd_id_fire, "FP_MeanT5", DFNT_FLOAT32, 1,
		dim_sizes)) == FAIL)
		return error(ERR_HDF);

	if (dim_sizes[0] != 0){
		if (SDsetchunk(sds_id, cdef, HDF_CHUNK | HDF_COMP) == FAIL)
			return error(ERR_HDF);

		if ((dim_id = SDgetdimid(sds_id, 0)) == FAIL) return ERR_HDF;
		if (SDsetdimname(dim_id, dimname1) == FAIL) return ERR_HDF;

		/* Write */	
		if (SDwritedata(sds_id, start, NULL, edges, fire_data->mean_T5) == FAIL)
			return ERR_HDF;
	}

	/* Add attributes */
	strcpy(long_name, "I05 brightness temperature of background");
	n_values = strlen(long_name);
	if (SDsetattr(sds_id, "long_name", DFNT_CHAR8, n_values, long_name))
		return error(ERR_HDF);

	strcpy(units, "kelvins");
	n_values = strlen(units);
	if (SDsetattr(sds_id, "units", DFNT_CHAR8, n_values, units))
		return error(ERR_HDF);

	if (SDendaccess(sds_id) == FAIL) return error(ERR_HDF);
	/* --- --- */

	/* --- mean_Dif45 --- */
	/* Init */
	out_info->sds_index_MeanDif = sds_index++;
	if ((sds_id = SDcreate(out_info->sd_id_fire, "FP_MeanDT", DFNT_FLOAT32, 1,
		dim_sizes)) == FAIL)
		return error(ERR_HDF);

	if (dim_sizes[0] != 0){
		if (SDsetchunk(sds_id, cdef, HDF_CHUNK | HDF_COMP) == FAIL)
			return error(ERR_HDF);

		if ((dim_id = SDgetdimid(sds_id, 0)) == FAIL) return ERR_HDF;
		if (SDsetdimname(dim_id, dimname1) == FAIL) return ERR_HDF;

		/* Write */	
		if (SDwritedata(sds_id, start, NULL, edges, fire_data->mean_Dif45) == FAIL)
			return ERR_HDF;
	}

	/* Add attributes */
	strcpy(long_name, "mean background brightness temperature difference");
	n_values = strlen(long_name);
	if (SDsetattr(sds_id, "long_name", DFNT_CHAR8, n_values, long_name))
		return error(ERR_HDF);

	strcpy(units, "kelvins");
	n_values = strlen(units);
	if (SDsetattr(sds_id, "units", DFNT_CHAR8, n_values, units))
		return error(ERR_HDF);

	if (SDendaccess(sds_id) == FAIL) return error(ERR_HDF);
	/* --- --- */

	/* --- MAD_T4 --- */
	/* Init */
	out_info->sds_index_MADT4 = sds_index++;
	if ((sds_id = SDcreate(out_info->sd_id_fire, "FP_MAD_T4", DFNT_FLOAT32, 1,
		dim_sizes)) == FAIL)
		return error(ERR_HDF);

	if (dim_sizes[0] != 0){
		if (SDsetchunk(sds_id, cdef, HDF_CHUNK | HDF_COMP) == FAIL)
			return error(ERR_HDF);

		if ((dim_id = SDgetdimid(sds_id, 0)) == FAIL) return ERR_HDF;
		if (SDsetdimname(dim_id, dimname1) == FAIL) return ERR_HDF;

		/* Write */	
		if (SDwritedata(sds_id, start, NULL, edges, fire_data->MAD_T4) == FAIL)
			return ERR_HDF;
	}

	/* Add attributes */
	strcpy(long_name, "background I04 brightness temperature mean absolute deviation");
	n_values = strlen(long_name);
	if (SDsetattr(sds_id, "long_name", DFNT_CHAR8, n_values, long_name))
		return error(ERR_HDF);

	strcpy(units, "kelvins");
	n_values = strlen(units);
	if (SDsetattr(sds_id, "units", DFNT_CHAR8, n_values, units))
		return error(ERR_HDF);

	if (SDendaccess(sds_id) == FAIL) return error(ERR_HDF);
	/* --- --- */

	/* --- MAD_T5 --- */
	/* Init */
	out_info->sds_index_MADT5 = sds_index++;
	if ((sds_id = SDcreate(out_info->sd_id_fire, "FP_MAD_T5", DFNT_FLOAT32, 1,
		dim_sizes)) == FAIL)
		return error(ERR_HDF);

	if (dim_sizes[0] != 0){
		if (SDsetchunk(sds_id, cdef, HDF_CHUNK | HDF_COMP) == FAIL)
			return error(ERR_HDF);

		if ((dim_id = SDgetdimid(sds_id, 0)) == FAIL) return ERR_HDF;
		if (SDsetdimname(dim_id, dimname1) == FAIL) return ERR_HDF;

		/* Write */	
		if (SDwritedata(sds_id, start, NULL, edges, fire_data->MAD_T5) == FAIL)
			return ERR_HDF;
	}

	/* Add attributes */
	strcpy(long_name, "background I05 brightness temperature mean absolute deviation");
	n_values = strlen(long_name);
	if (SDsetattr(sds_id, "long_name", DFNT_CHAR8, n_values, long_name))
		return error(ERR_HDF);

	strcpy(units, "kelvins");
	n_values = strlen(units);
	if (SDsetattr(sds_id, "units", DFNT_CHAR8, n_values, units))
		return error(ERR_HDF);

	if (SDendaccess(sds_id) == FAIL) return error(ERR_HDF);
	/* --- --- */



	/* --- MADDT --- */
	/* Init */
	out_info->sds_index_MADDif = sds_index++;
	if ((sds_id = SDcreate(out_info->sd_id_fire, "FP_MAD_DT", DFNT_FLOAT32, 1,
		dim_sizes)) == FAIL)
		return error(ERR_HDF);

	if (dim_sizes[0] != 0){
		if (SDsetchunk(sds_id, cdef, HDF_CHUNK | HDF_COMP) == FAIL)
			return error(ERR_HDF);

		if ((dim_id = SDgetdimid(sds_id, 0)) == FAIL) return ERR_HDF;
		if (SDsetdimname(dim_id, dimname1) == FAIL) return ERR_HDF;

		/* Write */	
		if (SDwritedata(sds_id, start, NULL, edges, fire_data->MAD_Dif45) == FAIL)
			return ERR_HDF;
	}

	/* Add attributes */
	strcpy(long_name, "background brightness temperature difference mean absolute deviation");
	n_values = strlen(long_name);
	if (SDsetattr(sds_id, "long_name", DFNT_CHAR8, n_values, long_name))
		return error(ERR_HDF);

	strcpy(units, "kelvins");
	n_values = strlen(units);
	if (SDsetattr(sds_id, "units", DFNT_CHAR8, n_values, units))
		return error(ERR_HDF);

	if (SDendaccess(sds_id) == FAIL) return error(ERR_HDF);
	/* --- --- */



	/* --- FRP --- */
	/* Init */
	out_info->sds_index_FRP = sds_index++;
	if ((sds_id = SDcreate(out_info->sd_id_fire, "FP_power", DFNT_FLOAT32, 1,
		dim_sizes)) == FAIL)
		return error(ERR_HDF);

	if (dim_sizes[0] != 0){
		if (SDsetchunk(sds_id, cdef, HDF_CHUNK | HDF_COMP) == FAIL)
			return error(ERR_HDF);

		if ((dim_id = SDgetdimid(sds_id, 0)) == FAIL) return ERR_HDF;
		if (SDsetdimname(dim_id, dimname1) == FAIL) return ERR_HDF;

		/* Write */	
		if (SDwritedata(sds_id, start, NULL, edges, fire_data->FRP) == FAIL)
			return ERR_HDF;
	}

	/* Add attributes */
	strcpy(long_name, "fire radiative power");
	n_values = strlen(long_name);
	if (SDsetattr(sds_id, "long_name", DFNT_CHAR8, n_values, long_name))
		return error(ERR_HDF);

	strcpy(units, "MW");
	n_values = strlen(units);
	if (SDsetattr(sds_id, "units", DFNT_CHAR8, n_values, units))
		return error(ERR_HDF);

	if (SDendaccess(sds_id) == FAIL) return error(ERR_HDF);
	/* --- --- */



	/* --- Adj Cloud --- */
	/* Init */
	out_info->sds_index_AdjCloud = sds_index++;
	if ((sds_id = SDcreate(out_info->sd_id_fire, "FP_AdjCloud", DFNT_UINT16, 1,
		dim_sizes)) == FAIL)
		return error(ERR_HDF);

	if (dim_sizes[0] != 0){
		if (SDsetchunk(sds_id, cdef, HDF_CHUNK | HDF_COMP) == FAIL)
			return error(ERR_HDF);

		if ((dim_id = SDgetdimid(sds_id, 0)) == FAIL) return ERR_HDF;
		if (SDsetdimname(dim_id, dimname1) == FAIL) return ERR_HDF;

		/* Write */	
		if (SDwritedata(sds_id, start, NULL, edges, fire_data->Adj_Cloud) == FAIL)
			return ERR_HDF;
	}

	/* Add attributes */
	strcpy(long_name, "number of adjacent cloud pixels");
	n_values = strlen(long_name);
	if (SDsetattr(sds_id, "long_name", DFNT_CHAR8, n_values, long_name))
		return error(ERR_HDF);

	if (SDendaccess(sds_id) == FAIL) return error(ERR_HDF);
	/* --- --- */


	/* --- Adj Water --- */
	/* Init */
	out_info->sds_index_AdjWater = sds_index++;
	if ((sds_id = SDcreate(out_info->sd_id_fire, "FP_AdjWater", DFNT_UINT16, 1,
		dim_sizes)) == FAIL)
		return error(ERR_HDF);

	if (dim_sizes[0] != 0){
		if (SDsetchunk(sds_id, cdef, HDF_CHUNK | HDF_COMP) == FAIL)
			return error(ERR_HDF);

		if ((dim_id = SDgetdimid(sds_id, 0)) == FAIL) return ERR_HDF;
		if (SDsetdimname(dim_id, dimname1) == FAIL) return ERR_HDF;

		/* Write */	
		if (SDwritedata(sds_id, start, NULL, edges, fire_data->Adj_Water) == FAIL)
			return ERR_HDF;
	}

	/* Add attributes */
	strcpy(long_name, "number of adjacent water pixels");
	n_values = strlen(long_name);
	if (SDsetattr(sds_id, "long_name", DFNT_CHAR8, n_values, long_name))
		return error(ERR_HDF);

	if (SDendaccess(sds_id) == FAIL) return error(ERR_HDF);
	/* --- --- */

	/* --- Window_Size --- */
	/* Init */
	out_info->sds_index_WindowSize = sds_index++;
	if ((sds_id = SDcreate(out_info->sd_id_fire, "FP_WinSize", DFNT_UINT16, 1,
		dim_sizes)) == FAIL)
		return error(ERR_HDF);

	if (dim_sizes[0] != 0){
		if (SDsetchunk(sds_id, cdef, HDF_CHUNK | HDF_COMP) == FAIL)
			return error(ERR_HDF);

		if ((dim_id = SDgetdimid(sds_id, 0)) == FAIL) return ERR_HDF;
		if (SDsetdimname(dim_id, dimname1) == FAIL) return ERR_HDF;

		/* Write */	
		if (SDwritedata(sds_id, start, NULL, edges, fire_data->Window_Size) == FAIL)
			return ERR_HDF;
	}

	/* Add attributes */
	strcpy(long_name, "background window size");
	n_values = strlen(long_name);
	if (SDsetattr(sds_id, "long_name", DFNT_CHAR8, n_values, long_name))
		return error(ERR_HDF);

	if (SDendaccess(sds_id) == FAIL) return error(ERR_HDF);
	/* --- --- */


	/* --- Confidence --- */
	/* Init */
	out_info->sds_index_Confidence = sds_index++;
	if ((sds_id = SDcreate(out_info->sd_id_fire, "FP_confidence", DFNT_UINT8, 1,
		dim_sizes)) == FAIL)
		return error(ERR_HDF);

	if (dim_sizes[0] != 0){
		if (SDsetchunk(sds_id, cdef, HDF_CHUNK | HDF_COMP) == FAIL)
			return error(ERR_HDF);

		if ((dim_id = SDgetdimid(sds_id, 0)) == FAIL) return ERR_HDF;
		if (SDsetdimname(dim_id, dimname1) == FAIL) return ERR_HDF;

		/* Write */	
		if (SDwritedata(sds_id, start, NULL, edges, fire_data->Mask) == FAIL)
			return ERR_HDF;
	}

	/* Add attributes */
	strcpy(long_name, "detection confidence");
	n_values = strlen(long_name);
	if (SDsetattr(sds_id, "long_name", DFNT_CHAR8, n_values, long_name))
		return error(ERR_HDF);

	if (SDendaccess(sds_id) == FAIL) return error(ERR_HDF);
	/* --- --- */
	

	/* --- Day --- */
	/* Init */
	out_info->sds_index_day = sds_index++;
	if ((sds_id = SDcreate(out_info->sd_id_fire, "FP_day", DFNT_UINT8, 1,
		dim_sizes)) == FAIL)
		return error(ERR_HDF);

	if (dim_sizes[0] != 0){
		if (SDsetchunk(sds_id, cdef, HDF_CHUNK | HDF_COMP) == FAIL)
			return error(ERR_HDF);

		if ((dim_id = SDgetdimid(sds_id, 0)) == FAIL) return ERR_HDF;
		if (SDsetdimname(dim_id, dimname1) == FAIL) return ERR_HDF;

		/* Write */	
		if (SDwritedata(sds_id, start, NULL, edges, fire_data->Day) == FAIL)
			return ERR_HDF;
	}

	/* Add attributes */
	strcpy(long_name, "day flag for fire pixel");
	n_values = strlen(long_name);
	if (SDsetattr(sds_id, "long_name", DFNT_CHAR8, n_values, long_name))
		return error(ERR_HDF);

	if (SDendaccess(sds_id) == FAIL) return error(ERR_HDF);
	/* --- --- */

	/* --- SolZenAng --- */
	/* Init */
	out_info->sds_index_SolZenAng = sds_index++;
	if ((sds_id = SDcreate(out_info->sd_id_fire, "FP_SolZenAng", DFNT_FLOAT32, 1,
		dim_sizes)) == FAIL)
		return error(ERR_HDF);

	if (dim_sizes[0] != 0){
		if (SDsetchunk(sds_id, cdef, HDF_CHUNK | HDF_COMP) == FAIL)
			return error(ERR_HDF);

		if ((dim_id = SDgetdimid(sds_id, 0)) == FAIL) return ERR_HDF;
		if (SDsetdimname(dim_id, dimname1) == FAIL) return ERR_HDF;

		/* Write */	
		if (SDwritedata(sds_id, start, NULL, edges, fire_data->SolZenAng) == FAIL)
			return ERR_HDF;
	}

	/* Add attributes */
	strcpy(long_name, "solar zenith angle of fire pixel");
	n_values = strlen(long_name);
	if (SDsetattr(sds_id, "long_name", DFNT_CHAR8, n_values, long_name))
		return error(ERR_HDF);

	strcpy(units, "degrees");
	n_values = strlen(units);
	if (SDsetattr(sds_id, "units", DFNT_CHAR8, n_values, units))
		return error(ERR_HDF);

	if (SDendaccess(sds_id) == FAIL) return error(ERR_HDF);
	/* --- --- */

	/* --- SolAzAng --- */
	/* Init */
	out_info->sds_index_SolAzAng = sds_index++;
	if ((sds_id = SDcreate(out_info->sd_id_fire, "FP_SolAzAng", DFNT_FLOAT32, 1,
		dim_sizes)) == FAIL)
		return error(ERR_HDF);

	if (dim_sizes[0] != 0){
		if (SDsetchunk(sds_id, cdef, HDF_CHUNK | HDF_COMP) == FAIL)
			return error(ERR_HDF);

		if ((dim_id = SDgetdimid(sds_id, 0)) == FAIL) return ERR_HDF;
		if (SDsetdimname(dim_id, dimname1) == FAIL) return ERR_HDF;

		/* Write */	
		if (SDwritedata(sds_id, start, NULL, edges, fire_data->SolAzAng) == FAIL)
			return ERR_HDF;
	}

	/* Add attributes */
	strcpy(long_name, "solar azimuth angle of fire pixel");
	n_values = strlen(long_name);
	if (SDsetattr(sds_id, "long_name", DFNT_CHAR8, n_values, long_name))
		return error(ERR_HDF);

	strcpy(units, "degrees");
	n_values = strlen(units);
	if (SDsetattr(sds_id, "units", DFNT_CHAR8, n_values, units))
		return error(ERR_HDF);

	if (SDendaccess(sds_id) == FAIL) return error(ERR_HDF);
	/* --- --- */

	/* --- ViewZenAng --- */
	/* Init */
	out_info->sds_index_ViewZenAng = sds_index++;
	if ((sds_id = SDcreate(out_info->sd_id_fire, "FP_ViewZenAng", DFNT_FLOAT32, 1,
		dim_sizes)) == FAIL)
		return error(ERR_HDF);
	
	if (dim_sizes[0] != 0){
		if (SDsetchunk(sds_id, cdef, HDF_CHUNK | HDF_COMP) == FAIL)
			return error(ERR_HDF);

		if ((dim_id = SDgetdimid(sds_id, 0)) == FAIL) return ERR_HDF;
		if (SDsetdimname(dim_id, dimname1) == FAIL) return ERR_HDF;

		/* Write */	
		if (SDwritedata(sds_id, start, NULL, edges, fire_data->ViewZenAng) == FAIL)
			return ERR_HDF;
	}

	/* Add attributes */
	strcpy(long_name, "view zenith angle of fire pixel");
	n_values = strlen(long_name);
	if (SDsetattr(sds_id, "long_name", DFNT_CHAR8, n_values, long_name))
		return error(ERR_HDF);

	strcpy(units, "degrees");
	n_values = strlen(units);
	if (SDsetattr(sds_id, "units", DFNT_CHAR8, n_values, units))
		return error(ERR_HDF);

	if (SDendaccess(sds_id) == FAIL) return error(ERR_HDF);
	/* --- --- */

	/* --- ViewAzAng --- */
	/* Init */
	out_info->sds_index_ViewAzAng = sds_index++;
	if ((sds_id = SDcreate(out_info->sd_id_fire, "FP_ViewAzAng", DFNT_FLOAT32, 1,
		dim_sizes)) == FAIL)
		return error(ERR_HDF);

	if (dim_sizes[0] != 0){
		if (SDsetchunk(sds_id, cdef, HDF_CHUNK | HDF_COMP) == FAIL)
			return error(ERR_HDF);

		if ((dim_id = SDgetdimid(sds_id, 0)) == FAIL) return ERR_HDF;
		if (SDsetdimname(dim_id, dimname1) == FAIL) return ERR_HDF;

		/* Write */	
		if (SDwritedata(sds_id, start, NULL, edges, fire_data->ViewAzAng) == FAIL)
			return ERR_HDF;
	}

	/* Add attributes */
	strcpy(long_name, "view azimuth angle of fire pixel");
	n_values = strlen(long_name);
	if (SDsetattr(sds_id, "long_name", DFNT_CHAR8, n_values, long_name))
		return error(ERR_HDF);

	strcpy(units, "degrees");
	n_values = strlen(units);
	if (SDsetattr(sds_id, "units", DFNT_CHAR8, n_values, units))
		return error(ERR_HDF);

	if (SDendaccess(sds_id) == FAIL) return error(ERR_HDF);
	/* --- --- */

	return 0;


}

/* Writes jugg->pixflag.mask to HDF output */
int WriteMask(OutputInfo *out_info, Juggernaut *jugg){

	int i, j;
	int32 sds_id;

	int32 start[2];
	int32 edges[2];

	start[0] = 0;
	edges[0] = jugg->number_of_scan_lines;

	start[1] = 0;
	edges[1] = SAMPLES_375M;

	uint8 **mask;
	if ( !(mask = (uint8**)malloc(jugg->number_of_scan_lines*sizeof(uint8*))) )
		return error(ERR_F_MEM);
	if ( !(mask[0] = (uint8*)malloc(jugg->number_of_scan_lines*SAMPLES_375M*sizeof(uint8))) )
		return error(ERR_F_MEM);
	for (i=1;i<jugg->number_of_scan_lines;i++){
		mask[i] = mask[i-1] + SAMPLES_375M;
	}

	uint32 **QA;
	if ( !(QA = (uint32**)malloc(jugg->number_of_scan_lines*sizeof(uint32*))) )
		return error(ERR_F_MEM);
	if ( !(QA[0] = (uint32*)malloc(jugg->number_of_scan_lines*SAMPLES_375M*sizeof(uint32))) )
		return error(ERR_F_MEM);
	for (i=1;i<jugg->number_of_scan_lines;i++){
		QA[i] = QA[i-1] + SAMPLES_375M;
	}


	for(i=0;i<jugg->number_of_scan_lines;i++){
		for(j=0;j<SAMPLES_375M;j++){
			mask[i][j] = jugg->pixflags[i][j].mask;
			QA[i][j] = jugg->pixflags[i][j].meta_data;
		}
	}

	start[0] = 0;
	edges[0] = jugg->number_of_scan_lines;

	start[1] = 0;
	edges[1] = SAMPLES_375M;

	

	/* Write fire mask */
	if ((sds_id = SDselect(out_info->sd_id_fire, out_info->sds_index_mask)) == FAIL)
		return ERR_HDF;
	if (SDwritedata(sds_id, start, NULL, edges, mask[0]) == FAIL)
		return ERR_HDF;
	if (SDendaccess(sds_id) == FAIL) return ERR_HDF;

	/* Write algorithm QA */
	if ((sds_id = SDselect(out_info->sd_id_fire, out_info->sds_index_qa)) == FAIL)
		return ERR_HDF;
	if (SDwritedata(sds_id, start, NULL, edges, QA[0]) == FAIL)
		return ERR_HDF;
	if (SDendaccess(sds_id) == FAIL) return ERR_HDF;


}

/* Fills in other FireData elements */
void PopulateFireData(Juggernaut *jugg, FireData *fire_data){
	
	int mask;
	int i, j, f, x, y;
	int cloud_count;
	int water_count;
	int redact_count = 0;

	for(f=0;f<fire_data->index;f++){
		i = fire_data->Row[f];
		j = fire_data->Col[f];

		mask = jugg->pixflags[i][j].mask;

		/* Fire was later rejected */
		if(mask != 7 && mask != 8 && mask != 9){
			redact_count++;
			continue;
		}


		fire_data->Mask[f-redact_count] = mask;

		fire_data->Row[f-redact_count] = i;
		fire_data->Col[f-redact_count] = j;
		

		fire_data->mean_T4[f-redact_count] = fire_data->mean_T4[f];
		fire_data->mean_T5[f-redact_count] = fire_data->mean_T5[f];
		fire_data->MAD_T4[f-redact_count] = fire_data->MAD_T4[f];
		fire_data->MAD_T5[f-redact_count] = fire_data->MAD_T5[f];
		fire_data->mean_Dif45[f-redact_count] = fire_data->mean_Dif45[f];
		fire_data->MAD_Dif45[f-redact_count] = fire_data->MAD_Dif45[f];
		fire_data->Window_Size[f-redact_count] = fire_data->Window_Size[f];


		fire_data->Lon[f-redact_count] = jugg->lon[i][j];
		fire_data->Lat[f-redact_count] = jugg->lat[i][j];


		fire_data->T4[f-redact_count] = jugg->T4[i][j];
		fire_data->T5[f-redact_count] = jugg->T5[i][j];


		fire_data->SolZenAng[f-redact_count] = jugg->SolZenAng[i][j];
		fire_data->SolAzAng[f-redact_count] = jugg->SolAzAng[i][j];
		fire_data->ViewZenAng[f-redact_count] = jugg->ViewZenAng[i][j];
		fire_data->ViewAzAng[f-redact_count] = jugg->ViewAzAng[i][j];

		fire_data->Day[f-redact_count] = jugg->pixflags[i][j].day;

		
		fire_data->FRP[f-redact_count] = 0;

		/* Count adj water and clouds */
		cloud_count = 0;
		water_count = 0;

		for(y=-1;y<=1;y++){
			for(x=-1;x<=1;x++){
				if(y == 0 && x == 0) continue;
				if(i+y < 0 || i+y >= jugg->number_of_scan_lines) continue;
				if(j+x < 0 || j+x >= SAMPLES_375M) continue;


				if(jugg->pixflags[i+y][j+x].water_mask == 1){
					water_count++;
				}
				if(jugg->pixflags[i+y][j+x].mask == 4){
					cloud_count++;
				}

			}
		}			
		
		fire_data->Adj_Cloud[f-redact_count] = cloud_count;
		fire_data->Adj_Water[f-redact_count] = water_count;	
	}
	fire_data->index = fire_data->index - redact_count;
	
}

/* Write post run global attributes */
void WriteGlobalAttr(OutputInfo *out_info, Juggernaut *jugg, FireData *fire_data){

	int day_night_both_flag = 0; // day=1,night=2,both=3
	char day_night_both_str[10];
	int n_values;
	int32 *temp_ptr;

	/* Counts */
	int total_day_pixels = jugg->day_count;
	int total_night_pixels = jugg->night_count;
	int total_missing_pixels = jugg->missing_count;
	int total_glint_pixels = jugg->glint_count;
	int total_land_pixels = jugg->land_count;
	int total_water_pixels = jugg->water_count;
	int total_cloud_pixels = jugg->cloud_count;
	int total_glintRejection_pixels = jugg->glint_rejection_count;


	/***** Global attributes *****/
	if(total_night_pixels == 0){
		strcpy(day_night_both_str, "Day");
	}else if(total_day_pixels == 0){
		strcpy(day_night_both_str, "Night");
	}else{
		strcpy(day_night_both_str, "Both");
	}

	n_values = strlen(day_night_both_str);
	if (SDsetattr(out_info->sd_id_fire, "Day/Night/Both", DFNT_CHAR8, n_values, day_night_both_str))
		return error(ERR_HDF);

	temp_ptr = &fire_data->index;
	if (SDsetattr(out_info->sd_id_fire, "FirePix", DFNT_INT32, 1, temp_ptr))
		return error(ERR_HDF);
	
	temp_ptr = &total_day_pixels;
	if (SDsetattr(out_info->sd_id_fire, "DayPix", DFNT_INT32, 1, temp_ptr))
		return error(ERR_HDF);

	temp_ptr = &total_night_pixels;
	if (SDsetattr(out_info->sd_id_fire, "NightPix", DFNT_INT32, 1, temp_ptr))
		return error(ERR_HDF);

	temp_ptr = &total_land_pixels;
	if (SDsetattr(out_info->sd_id_fire, "LandPix", DFNT_INT32, 1, temp_ptr))
		return error(ERR_HDF);

	temp_ptr = &total_water_pixels;
	if (SDsetattr(out_info->sd_id_fire, "WaterPix", DFNT_INT32, 1, temp_ptr))
		return error(ERR_HDF);

	temp_ptr = &total_missing_pixels;
	if (SDsetattr(out_info->sd_id_fire, "MissingPix", DFNT_INT32, 1, temp_ptr))
		return error(ERR_HDF);

	temp_ptr = &total_glint_pixels;
	if (SDsetattr(out_info->sd_id_fire, "GlintPix", DFNT_INT32, 1, temp_ptr))
		return error(ERR_HDF);

	temp_ptr = &total_cloud_pixels;
	if (SDsetattr(out_info->sd_id_fire, "CloudPix", DFNT_INT32, 1, temp_ptr))
		return error(ERR_HDF);

	temp_ptr = &total_glintRejection_pixels;
	if (SDsetattr(out_info->sd_id_fire, "GlintRejectedPix", DFNT_INT32, 1, temp_ptr))
		return error(ERR_HDF);

	/*****  		*****/


}

/* Writes the header for the ascii file*/
void InitOutputASCII(OutputInfo *out_info, FireData *fire_data){
	int len, date_pos;
	int offset = 67;

	/* Header */
	char VIIRS_Granule[20];
	strcpy(VIIRS_Granule, "VIIRS_Granule");
	char Scan_Time[12];
	strcpy(Scan_Time, "Scan_Time");
	char Lon[4];
	strcpy(Lon, "Lon");
	char Lat[4];
	strcpy(Lat, "Lat");
	char Sample[7];
	strcpy(Sample, "Sample");
	char Line[5];
	strcpy(Line, "Line");
	char BT_I4[9];
	strcpy(BT_I4, "BT_I4(K)");
	char BT_I5[9];
	strcpy(BT_I5, "BT_I5(K)");
	char Confidence[15];
	strcpy(Confidence, "Confidence");
	char Period[7];
	strcpy(Period, "Period");

	/* Getting the granule time */
	len = strlen(out_info->svi01_name);
	date_pos = len - offset;
	strncpy(fire_data->Granule, out_info->svi01_name + date_pos, 26);
		fire_data->Granule[26] = '\0';

	/* Opening file and writing first line */ 
	out_info->ascii_ptr = fopen(out_info->ascii_name, "w");

	fprintf(out_info->ascii_ptr, "%27s,%18s,%11s,%10s,%7s,%7s,%9s,%9s,%11s,%7s\n", &VIIRS_Granule, &Scan_Time, &Lon, &Lat, &Sample, &Line, &BT_I4, &BT_I5, &Confidence, &Period);
	fflush(out_info->ascii_ptr);

	return 0;
}

/* Sets the dimension names */
static int set_dimensions(int32 sds_id, char *dimname1, char *dimname2){
	int32 dim_id;

	if ((dim_id = SDgetdimid(sds_id, 0)) == FAIL) return ERR_HDF;
	if (SDsetdimname(dim_id, dimname1) == FAIL) return ERR_HDF;
	if ((dim_id = SDgetdimid(sds_id, 1)) == FAIL) return ERR_HDF;
	if (SDsetdimname(dim_id, dimname2) == FAIL) return ERR_HDF;

	return 0;
}
