/*
!Description: vfire-2.x.x reads the entire input data into on big struct.


!References and Credits:

	W. Schroeder, University of Maryland
	W. Walsh, University of Maryland
	
	Some of the framework was taken from M-band active fires, May 2012, L. Giglio.

	Last Edit: February 2016

!Design Notes:

*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <unistd.h>
#include <time.h>

#include "hdf5.h"


#include "const.h"
#include "types.h"
#include "output.h"
#include "input.h"
#include "process.h"
#include "error.h"
#include "version.h"


int main(int argc, char **argv)
{
	clock_t start = clock();
	int i;

	hid_t sd_i01;
	hid_t sd_i02;
	hid_t sd_i03;
	hid_t sd_i04;
	hid_t sd_i05;
	hid_t sd_gitco;
	hid_t sd_m13_agg;
	hid_t sd_m13_unagg;
	int32 sd_fire;

	char *infile_i01;	/* I-band #1 */
	char *infile_i02;	/* I-band #2 */
	char *infile_i03;	/* I-band #3 */
	char *infile_i04;	/* I-band #4 */
	char *infile_i05;	/* I-band #5 */
	char *infile_gitco;	/* geo */
	char *infile_m13_agg;	/* M-band 13 agg */
	char *infile_m13_unagg; /* M-band 13 unagg */

	char *slash_index;
	int filename_offset;
	int filename_length;

	char input_files[8][100];

	InputInfo in_info;
	OutputInfo out_info;


	char *outfile;
	char *ascii_file;
	int st;

	char msg[MAX_MSG_LEN];

	int c;
	extern char *optarg;
	extern int optind, opterr;

	int verbose;
	int write_ascii;		
	int write_hdf;
	int agg_13;
	int unagg_13;


	char *temp_ptr;
	int n_values;


	/* Command line flags */
	verbose = 0;
	write_ascii = write_hdf = 0;
	agg_13 = unagg_13 = 0;

	/* Read command line arguements */
	while ((c = getopt(argc, argv, "a:n:m:dv")) != -1) {
		switch (c) {
			case 'v':
				verbose = 1;
				break;
			case 'a':
				write_ascii = 1;
				ascii_file = optarg;
				break;

			case 'd':
				write_hdf = 1;
				break;
			case 'n':
				agg_13 = 1;
				infile_m13_agg = optarg;
				break;
			case 'm':
				unagg_13 = 1;
				infile_m13_unagg = optarg;
				break;
			default:
				USAGE:
				fputs("Usage: vfire375 [-a ascii_filename -dv] I01_file I02_file I03_file I04_file I05_file "
					"GITCO_file hdf_output (-n M13_agg_file -m M13_unagg_file)\n",
					stderr);
				fputs("    a	Write fire pixel data to ASCII file.\n", stderr);
				fputs("    d	Append fire pixel SDSs (sparse arrays) to product.\n", stderr);
				fputs("    n	Aggregated M13 (Mandatory for nighttime passes over South America).\n", stderr);
				fputs("    m	Unaggregated M13 (Mandatory for nighttime passes over South America).\n", stderr);
				fputs("    v	Verbose mode.\n", stderr);
				fprintf(stderr, "\nVersion %s, compiled %s %s.\n",
					PROCESS_VERSION_NUMBER, __DATE__, __TIME__);
				exit(1);
				
				break;
		}
	}

	/* Make sure all input file names are there */
	if (optind != argc - 7) goto USAGE;
	infile_i01 = argv[optind];
	infile_i02 = argv[optind+1];
	infile_i03 = argv[optind+2];
	infile_i04 = argv[optind+3];
	infile_i05 = argv[optind+4];
	infile_gitco = argv[optind+5];
	outfile = argv[optind+6];

	/* Getting filenames for global attribute */
	slash_index = strrchr(infile_i01, '/');
	if (slash_index == NULL){
		strcpy(input_files[0], infile_i01);
	}else{
		filename_offset = slash_index-infile_i01+1;
		strcpy(input_files[0], infile_i01+filename_offset);
	}

	slash_index = strrchr(infile_i02, '/');
	if (slash_index == NULL){
		strcpy(input_files[1], infile_i02);
	}else{
		filename_offset = slash_index-infile_i02+1;
		strcpy(input_files[1], infile_i02+filename_offset);
	}

	slash_index = strrchr(infile_i03, '/');
	if (slash_index == NULL){
		strcpy(input_files[2], infile_i03);
	}else{
		filename_offset = slash_index-infile_i03+1;
		strcpy(input_files[2], infile_i03+filename_offset);
	}

	slash_index = strrchr(infile_i04, '/');
	if (slash_index == NULL){
		strcpy(input_files[3], infile_i04);
	}else{
		filename_offset = slash_index-infile_i04+1;
		strcpy(input_files[3], infile_i04+filename_offset);
	}

	slash_index = strrchr(infile_i05, '/');
	if (slash_index == NULL){
		strcpy(input_files[4], infile_i05);
	}else{
		filename_offset = slash_index-infile_i05+1;
		strcpy(input_files[4], infile_i05+filename_offset);
	}

	slash_index = strrchr(infile_gitco, '/');
	if (slash_index == NULL){
		strcpy(input_files[5], infile_gitco);
	}else{
		filename_offset = slash_index-infile_gitco+1;
		strcpy(input_files[5], infile_gitco+filename_offset);
	}

	if(agg_13 == 1){
		slash_index = strrchr(infile_m13_agg, '/');
		if (slash_index == NULL){
			strcpy(input_files[6], infile_m13_agg);
		}else{
			filename_offset = slash_index-infile_m13_agg+1;
			strcpy(input_files[6], infile_m13_agg+filename_offset);
		}

		slash_index = strrchr(infile_m13_unagg, '/');
		if (slash_index == NULL){
			strcpy(input_files[7], infile_m13_unagg);
		}else{
			filename_offset = slash_index-infile_m13_unagg+1;
			strcpy(input_files[7], infile_m13_unagg+filename_offset);
		}
	}else{
		strcpy(input_files[6], "NULL");
		strcpy(input_files[7], "NULL");
	}
	/*    */

	/* Create file ids */
	if ((sd_i01 = H5Fopen(infile_i01, H5F_ACC_RDONLY, H5P_DEFAULT)) == FAIL) {
		(void) error(ERR_HDF_QUIET);
		(void) sprintf(msg, "Unable to open HDF input file \"%s\".",
			infile_i01);
		errmsg(msg);
		exit(1);
		}

	if ((sd_i02 = H5Fopen(infile_i02, H5F_ACC_RDONLY, H5P_DEFAULT)) == FAIL) {
		(void) error(ERR_HDF_QUIET);
		(void) sprintf(msg, "Unable to open HDF input file \"%s\".",
			infile_i02);
		errmsg(msg);
		exit(1);
		}

	if ((sd_i03 = H5Fopen(infile_i03, H5F_ACC_RDONLY, H5P_DEFAULT)) == FAIL) {
		(void) error(ERR_HDF_QUIET);
		(void) sprintf(msg, "Unable to open HDF input file \"%s\".",
			infile_i03);
		errmsg(msg);
		exit(1);
		}

	if ((sd_i04 = H5Fopen(infile_i04, H5F_ACC_RDONLY, H5P_DEFAULT)) == FAIL) {
		(void) error(ERR_HDF_QUIET);
		(void) sprintf(msg, "Unable to open HDF input file \"%s\".",
			infile_i04);
		errmsg(msg);
		exit(1);
		}

	if ((sd_i05 = H5Fopen(infile_i05, H5F_ACC_RDONLY, H5P_DEFAULT)) == FAIL) {
		(void) error(ERR_HDF_QUIET);
		(void) sprintf(msg, "Unable to open HDF input file \"%s\".",
			infile_i05);
		errmsg(msg);
		exit(1);
		}

	if ((sd_gitco = H5Fopen(infile_gitco, H5F_ACC_RDONLY, H5P_DEFAULT)) == FAIL) {
		(void) error(ERR_HDF_QUIET);
		(void) sprintf(msg, "Unable to open HDF input file \"%s\".",
			infile_gitco);
		errmsg(msg);
		exit(1);
		}

	if(agg_13 == 1){
		if ((sd_m13_agg = H5Fopen(infile_m13_agg, H5F_ACC_RDONLY, H5P_DEFAULT)) == FAIL) {
			(void) error(ERR_HDF_QUIET);
			(void) sprintf(msg, "Unable to open HDF input file \"%s\".",
				infile_m13_agg);
			errmsg(msg);
			exit(1);
			}
	}else{
		sd_m13_agg = NULL;
	}

	if(unagg_13 == 1){
		if ((sd_m13_unagg = H5Fopen(infile_m13_unagg, H5F_ACC_RDONLY, H5P_DEFAULT)) == FAIL) {
			(void) error(ERR_HDF_QUIET);
			(void) sprintf(msg, "Unable to open HDF input file \"%s\".",
				infile_m13_unagg);
			errmsg(msg);
			exit(1);
			}
	}else{
		sd_m13_unagg = NULL;
	}


	/* Initalize the input -- Stores the sds indexs in in_info struct */
	if ( (st = InitInputHDF(sd_i01, sd_i02, sd_i03, sd_i04, sd_i05, sd_gitco, sd_m13_agg, sd_m13_unagg, &in_info)) ) exit(st);



	/* Initalize the output -- Stores the sds indexs in out_info struct (adds fire_mask and algo qa) */
	st = InitOutputHDF(outfile, in_info.nscans, &out_info, input_files, write_ascii, write_hdf, ascii_file, sd_gitco);
	if (st) exit(4);



	/* Main function */
	st = ProcessInput(&in_info, &out_info);
	if (st) {
		fputs("Error processing data.\n", stderr);
		exit(3);
	}



	/* Close files */
	//if (H5Fclose(sd_fire) == FAIL) exit(error(ERR_HDF));
	if (H5Fclose(sd_i01) == FAIL) exit(error(ERR_HDF));
	if (H5Fclose(sd_i02) == FAIL) exit(error(ERR_HDF));
	if (H5Fclose(sd_i03) == FAIL) exit(error(ERR_HDF));
	if (H5Fclose(sd_i04) == FAIL) exit(error(ERR_HDF));
	if (H5Fclose(sd_i05) == FAIL) exit(error(ERR_HDF));
	if (H5Fclose(sd_gitco) == FAIL) exit(error(ERR_HDF));
	if(sd_m13_agg != NULL){
		if (H5Fclose(sd_m13_agg) == FAIL) exit(error(ERR_HDF));
	}
	if(sd_m13_unagg != NULL){
		if (H5Fclose(sd_m13_unagg) == FAIL) exit(error(ERR_HDF));
	}

	/* Close output */
	SDend (sd_fire);


	clock_t end = clock();
	double diff = (double)(end - start) / CLOCKS_PER_SEC;
	if(verbose) printf("Time: %lf\n", diff);
	
	exit(0);
}

